% Input matrices for Example on pp. 17-21 of the Balvers and Mitchell working paper.
% Note that the problem is controllable and observable in the Chow form and
% therefore its state vector is minimal at a size of 2. Nevertheless, the Riccati kernel
% is smaller, equal to 1, and the problem has a simple analytical solution.

aa=2
bb=20
cc=5
dd=2
rho=0.2
hh=0.5
qq=1

Abar=[1 0 0 0;-bb/cc 1+(1/aa) 0 (1/cc) + (1/aa); 0 1 0 0; -dd*bb/cc dd*(1-rho)+(dd/aa) 0 rho+(dd/cc)+(dd/aa)]
Cbar=[0; -(1/aa); 0; -(dd/aa)]
Kbar=[qq^2 0 0 -qq; 0 hh -hh 0; 0 -hh hh 0; -qq 0 0 1]
G=[0 0; 0 1; 0 0; 1 dd];
J=zeros(4,1);
R=zeros(1);
u0bar=zeros(1);
Dbar=[-qq 0 0 1; 0 (hh^0.5) -(hh^0.5) 0]
D=[Dbar zeros(2,1)]
y0bar=[0; 0; 0; 0]
T=6
b=1;
c=1;
alpha=1+(1/aa)
gamma=(1/cc) + (1/aa)
delta=dd

% Equation (37) on p.21 of Balvers and Mitchell gives the closed-form
% solution for steady state Phi, which becomes for the current numbers:

PhiSolved=inv((1+sqrt(1+(4*hh/(dd^2))))/(2*((1-alpha)^2)*hh))

% Equation (36) provides the optimal feedback response to lagged price level
% and current output as a linear function of Phi:

FSolved= [1 alpha+gamma*dd]*[alpha-1+gamma*dd + PhiSolved/(hh*(1-alpha)^2)]
    

