% The user must input matrices Kbar (state cost matrix), Abar (transition matrix)
% Cbar (control multiplier matrix), J (crossproduct matrix), R (control cost matrix)
% and scalars b (discount factor), c (weight on final period state cost), T (horizon)
% Note that all variables definitions are case sensitive.
% Kbar: n by n
% Abar: n by n
% Cbar: n by k
% J: 	n by k
% R: 	k by k
% b:    1 by 1
% c: 	1 by 1
% T: 	1 by 1
% Output is the size of the Riccati kernel and the full feedback and
% Riccati matrices.

KTbar=c*Kbar;
JT=c*J;
RT=c*R;
[nbar,kbar]=size(Cbar);

K=[Kbar J; J' R];
KT=[KTbar JT; JT' RT];
A=sqrt(b)*[Abar zeros(nbar,kbar); zeros(kbar,nbar) zeros(kbar)];
C=sqrt(b)*[Cbar; eye(kbar)];
[n,k]=size(C);

[Ac, Cc, Kc, S]=RedAC(A,C,K);
[nc,kc]=size(Cc);

timediff=0
[Ak,Kk,Ck,Rk,TransfoK]=RedK(Ac,Cc,Kc,c);
KTk=c*Kk;
[nk,kk]=size(Ck);
if nk<nc
    timediff=timediff+1
end

Aq=Ak; Kq=Kk; Cq=Ck; Rq=Rk; KTq=KTk;

nq=nk
kq=kk

Transfo2star=zeros(kq,nq);
Transfo1star=eye(nq);
Tr=zeros(nq);

atimediff=timediff
while rank(Aq)<nq
    [Ad,Cd,Kd,Rd,Transfo1,Transfo2]=RedA(Aq,Cq,Kq,Rq,c)
    Tr=Tr+Kq
    Aq=Ad; Cq=Cd; Kq=Kd; Rq=Rd;
    Transfo2star=Transfo2*Transfo1 + Transfo2star*Transfo1star;
    Transfo1star=Transfo1*Transfo1star;
    [nq,kq]=size(Cq)
    atimediff=atimediff+1
end
Aa=Aq; Ca=Cq; Ka=Kq; Ra=Rq; KTa=c*Kq;

[Ha,Fa]=BasicRiccati(Aa,Ca,Ka,KTa,Ra,T);

for t=1:T-atimediff
    Fk(:,:,t+timediff)=Transfo2star + Fa(:,:,t+atimediff)*Transfo1star;
    Hk(:,:,t+timediff)=Transfo1star'*Ha(:,:,t+atimediff)*Transfo1star + Tr;
    Fc(:,:,t)=Fk(:,:,t+timediff)*TransfoK + inv(Cc'*Kc*Cc)*Cc'*Kc*Ac;
    Hc(:,:,t)=TransfoK'*Hk(:,:,t+timediff)*TransfoK + Kc;
end

[H,F]=BasicRiccati(A,C,K,KT,zeros(k),T);

for t=1:T-atimediff
    Flong(:,:,t)=F(:,:,t)*S
    Hlong(:,:,t)=S'*H(:,:,t)*S
    Fshort(:,:,t)=Fc(:,:,t)
    Hshort(:,:,t)=Hc(:,:,t)
end

% ending of file RedMainFin.m
% Copyright July 2003, Ronald J. Balvers, West Virginia University.